using Ato.CD.Inbound.Shared;
using Ato.CD.Inbound.Shared201802;
using Ato.EN.IntegrationServices.CodeGenerationCGTS;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using DataContracts;
using System.Collections.Generic;
using System.Linq;

namespace Ato.CD.Inbound.TRT202402
{
    public class CrossFormValidatorCGTS : ICrossFormValidator
    {
        private CGTS2018 childDocument { get; set; }

        private List<Context> contexts { get; set; }

        public List<ProcessMessageDocument> response { get; private set; }

        private TRT2024 parentDocument { get; set; }

        public CrossFormValidatorCGTS(TRT2024 parent, object child)
        {
            childDocument = (CGTS2018)child;
            contexts = childDocument.GetContexts();
            parentDocument = parent;
            response = new List<ProcessMessageDocument>();
        }

        public List<ProcessMessageDocument> ValidateCrossFormRules()
        {
            VRATOTRT432189();
            VRATOTRT433043();
            VRATOTRT433044();
            VRATOTRT433045();
            VRATOTRT433046();
            VRATOTRT433047();
            VRATOTRT433048();
            VRATOTRT433049();
            VRATOTRT432213();

            return response;
        }

        #region Validation Rules

        #region VR.ATO.TRT.432189

        /*  VR.ATO.TRT.432189
                    The Net Capital gain amount on the main return form must equal the Total net capital gain amount on the Capital Gains Tax schedule.

                Legacy Rule Format:
                    CountDocument('CGTS') = 1 AND ^TRT133 <> ^CGTS131

            Technical Business Rule Format:
                    CountDocument('CGTS') = 1 AND ^TRT133 <> ^CGTS131

        Data Elements:
    
            ^TRT133 = TRT:RP:BusinessDetails:Income:CapitalGains:Income.CapitalGainsNet.Amount
    
            ^CGTS131 = CGTS131
            */
        public virtual void VRATOTRT432189()
        {
            bool assertion = parentDocument.TRT133.GetValueOrDefault() != childDocument.CGTS131.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gains in return must equal total net capital gain in CGT schedule.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:NetA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432189" },
                        new ProcessMessageParameter() { Name = "TRT133", Value = parentDocument.TRT133.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "CGTS131", Value = childDocument.CGTS131.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432189

        #region VR.ATO.TRT.433043

        /*  VR.ATO.TRT.433043
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    ^TRT448 = 'A - Small business active asset reduction' AND CountDocument('CGTS') = 1 AND ^CGTS127 = NULL

            Technical Business Rule Format:
                    ^TRT448 = 'A - Small business active asset reduction' AND CountDocument('CGTS') = 1 AND ^CGTS127 = NULL

        Data Elements:
    
            ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS127 = CGTS127
            */
        public virtual void VRATOTRT433043()
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"A - Small business active asset reduction")) && childDocument.CGTS127 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business active asset reduction amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433043" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "A - Small business active asset reduction" },
                        new ProcessMessageParameter() { Name = "CGTS127", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433043

        #region VR.ATO.TRT.433044

        /*  VR.ATO.TRT.433044
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business retirement exemption (Subdivision152-D)", then Small business retirement exemption amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    ^TRT448 = 'B - Small business retirement exemption' AND CountDocument('CGTS') = 1 AND ^CGTS128 = NULL

            Technical Business Rule Format:
                    ^TRT448 = 'B - Small business retirement exemption' AND CountDocument('CGTS') = 1 AND ^CGTS128 = NULL

        Data Elements:
    
            ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS128 = CGTS128
            */
        public virtual void VRATOTRT433044()
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"B - Small business retirement exemption")) && childDocument.CGTS128 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business retirement exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business retirement exemption (Subdivision152-D)"", then Small business retirement exemption amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433044" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "B - Small business retirement exemption" },
                        new ProcessMessageParameter() { Name = "CGTS128", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433044

        #region VR.ATO.TRT.433045

        /*  VR.ATO.TRT.433045
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    ^TRT448 = 'C - Small business roll-over' AND CountDocument('CGTS') = 1 AND ^CGTS129 = NULL

            Technical Business Rule Format:
                    ^TRT448 = 'C - Small business roll-over' AND CountDocument('CGTS') = 1 AND ^CGTS129 = NULL

        Data Elements:
    
            ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS129 = CGTS129
            */
        public virtual void VRATOTRT433045()
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"C - Small business roll-over")) && childDocument.CGTS129 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438024",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433045" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "C - Small business roll-over" },
                        new ProcessMessageParameter() { Name = "CGTS129", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433045

        #region VR.ATO.TRT.433046

        /*  VR.ATO.TRT.433046
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    ^TRT448 = 'D - Small business 15 year exemption' AND CountDocument('CGTS') = 1 AND ^CGTS138 = NULL

            Technical Business Rule Format:
                    ^TRT448 = 'D - Small business 15 year exemption' AND CountDocument('CGTS') = 1 AND ^CGTS138 = NULL

        Data Elements:
    
            ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS138 = CGTS138
            */
        public virtual void VRATOTRT433046()
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"D - Small business 15 year exemption")) && childDocument.CGTS138 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business 15 year exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433046" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "D - Small business 15 year exemption" },
                        new ProcessMessageParameter() { Name = "CGTS138", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433046

        #region VR.ATO.TRT.433047

        /*  VR.ATO.TRT.433047
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Foreign resident CGT exemption (Division 855)", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    ^TRT448 = 'E - Foreign resident CGT exemption' AND CountDocument('CGTS') = 1 AND ^CGTS140 = NULL

            Technical Business Rule Format:
                    ^TRT448 = 'E - Foreign resident CGT exemption' AND CountDocument('CGTS') = 1 AND ^CGTS140 = NULL

        Data Elements:
    
            ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS140 = CGTS140
            */
        public virtual void VRATOTRT433047()
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"E - Foreign resident CGT exemption")) && childDocument.CGTS140 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438026",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a foreign resident amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Foreign resident CGT exemption (Division 855)"", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433047" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "E - Foreign resident CGT exemption" },
                        new ProcessMessageParameter() { Name = "CGTS140", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433047

        #region VR.ATO.TRT.433048

        /*  VR.ATO.TRT.433048
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    ^TRT448 = 'F - Scrip for scrip roll-over' AND CountDocument('CGTS') = 1 AND ^CGTS45 = NULL

            Technical Business Rule Format:
                    ^TRT448 = 'F - Scrip for scrip roll-over' AND CountDocument('CGTS') = 1 AND ^CGTS45 = NULL

        Data Elements:
    
            ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS45 = CGTS45
            */
        public virtual void VRATOTRT433048()
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"F - Scrip for scrip roll-over")) && childDocument.CGTS45 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438027",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433048" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "F - Scrip for scrip roll-over" },
                        new ProcessMessageParameter() { Name = "CGTS45", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433048

        #region VR.ATO.TRT.433049

        /*  VR.ATO.TRT.433049
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Demerger exemption (Subdivision 125-C)", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    ^TRT448 = 'H - Demerger exemption' AND CountDocument('CGTS') = 1 AND ^CGTS47 = NULL

            Technical Business Rule Format:
                    ^TRT448 = 'H - Demerger exemption' AND CountDocument('CGTS') = 1 AND ^CGTS47 = NULL

        Data Elements:
    
            ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS47 = CGTS47
            */
        public virtual void VRATOTRT433049()
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"H - Demerger exemption")) && childDocument.CGTS47 == null;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438028",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a demerging entity amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Demerger exemption (Subdivision 125-C)"", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433049" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "H - Demerger exemption" },
                        new ProcessMessageParameter() { Name = "CGTS47", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433049

        #region VR.ATO.TRT.432213

        /*  VR.ATO.TRT.432213
                    Unapplied net capital losses carried forward on the Capital Gains Tax schedule (if lodged) must equal Net capital losses carried forward on the main income tax return.

                Legacy Rule Format:
                    CountDocument('CGTS') = 1 AND (^CGTS124 + ^CGTS125) <> ^TRT475

            Technical Business Rule Format:
                    CountDocument('CGTS') = 1 AND (^CGTS124 + ^CGTS125) <> ^TRT475

        Data Elements:
    
            ^TRT475 = TRT:RP:BusinessDetails:Income:Capital.Losses.CarriedForward.Net.Amount
    
            ^CGTS124 = CGTS124
    
            ^CGTS125 = CGTS125
            */
        public virtual void VRATOTRT432213()
        {
            bool assertion = (childDocument.CGTS124.GetValueOrDefault() + childDocument.CGTS125.GetValueOrDefault()) != parentDocument.TRT475.GetValueOrDefault();
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434088",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in CGT schedule",
                    LongDescription = @"The total of the 'Unapplied net capital losses carried forward'  in the Capital Gains Tax schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalLossesCarriedForwardNetA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432213" },
                        new ProcessMessageParameter() { Name = "CGTS124", Value = childDocument.CGTS124.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "CGTS125", Value = childDocument.CGTS125.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT475", Value = parentDocument.TRT475.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432213

        #endregion Validation Rules
    }
}