using Ato.CD.Inbound.Shared;
using Ato.EN.IntegrationServices.CodeGenerationLS;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using DataContracts;
using System.Collections.Generic;

namespace Ato.CD.Inbound.TRT202402
{
    public class CrossFormValidatorLS : ICrossFormValidator
    {
        private LS2017 childDocument { get; set; }
        
        public List<ProcessMessageDocument> response { get; private set; }

        private TRT2024 parentDocument { get; set; }

        public CrossFormValidatorLS(TRT2024 parent, object child)
        {
            childDocument = (LS2017)child;
            parentDocument = parent;
            response = new List<ProcessMessageDocument>();
        }

        public List<ProcessMessageDocument> ValidateCrossFormRules()
        {
            VRATOTRT432208();
            VRATOTRT432212();
            VRATOTRT432215();

            return response;
        }

        #region Validation Rules

        #region VR.ATO.TRT.432208

        /*  VR.ATO.TRT.432208
                    Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities. Trusts are required to indicate whether continuity of majority ownership test passed for each year of loss.

                Legacy Rule Format:
                    ^TRT32 = '179' AND ^TRT144 > 100000 AND CountDocument('LS') = 1 AND ^LS15 = NULL AND ^LS16 = NULL AND ^LS17 = NULL AND ^LS18 = NULL AND ^LS19 = NULL AND ^LS53 = NULL

            Technical Business Rule Format:
                    ^TRT32 = '179' AND ^TRT144 > 100000 AND CountDocument('LS') = 1 AND ^LS15 = NULL AND ^LS16 = NULL AND ^LS17 = NULL AND ^LS18 = NULL AND ^LS19 = NULL AND ^LS53 = NULL

        Data Elements:
    
            ^TRT144 = TRT:RP:BusinessDetails:Income:TaxLossesDeducted:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^LS15 = LS15
    
            ^LS16 = LS16
    
            ^LS17 = LS17
    
            ^LS18 = LS18
    
            ^LS19 = LS19
    
            ^LS53 = LS53
    
            ^TRT32 = TRT:RP:TypeOfTrust:OrganisationDetails.OrganisationType.Code
            */
        public virtual void VRATOTRT432208()
        {
            bool assertion = parentDocument.TRT32 == @"179" && parentDocument.TRT144.GetValueOrDefault() > 100000 && childDocument.LS15 == null && childDocument.LS16 == null && childDocument.LS17 == null && childDocument.LS18 == null && childDocument.LS19 == null && childDocument.LS53 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432208",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Widely held trusts must indicate whether continuity of majority ownership test passed for each year of loss",
                    LongDescription = @"Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:TaxLossesDeductedTaxDeductionA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432208" },
                        new ProcessMessageParameter() { Name = "TRT144", Value = parentDocument.TRT144.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS15", Value = childDocument.LS15.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS16", Value = childDocument.LS16.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS17", Value = childDocument.LS17.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS18", Value = childDocument.LS18.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS19", Value = childDocument.LS19.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS53", Value = childDocument.LS53.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT144", Value = "Null" },
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432208

        #region VR.ATO.TRT.432212

        /*  VR.ATO.TRT.432212
                    Total of the tax losses carried forward to later income years in the losses schedule for unconsolidated entities must equal tax losses carried forward to later income years on the main income tax return.

                Legacy Rule Format:
                    CountDocument('LS') = 1 AND ^LS50 <> ^TRT145

            Technical Business Rule Format:
                    CountDocument('LS') = 1 AND ^LS50 <> ^TRT145

        Data Elements:
    
            ^TRT145 = TRT:RP:BusinessDetails:Income:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
    
            ^LS50 = LS50
            */
        public virtual void VRATOTRT432212()
        {
            bool assertion = childDocument.LS50.GetValueOrDefault() != parentDocument.TRT145.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax losses carried forward to later income years in schedule must equal amount in main form",
                    LongDescription = @"The total of the 'Tax losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Tax losses carried forward to later income years' on the main form",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:TaxLossesCarriedForwardLaterIncomeYearsTotalA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432212" },
                        new ProcessMessageParameter() { Name = "TRT145", Value = parentDocument.TRT145.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS50", Value = childDocument.LS50.GetValueOrDefault().ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432212

        #region VR.ATO.TRT.432215

        /*  VR.ATO.TRT.432215
                    Net capital losses carried forward to later income years - Total in the Losses schedule for unconsolidated entities is equal to the Net capital losses carried forward to later income years on the main income tax return.

                Legacy Rule Format:
                    CountDocument('LS') = 1 AND ^LS52 <> ^TRT475

            Technical Business Rule Format:
                    CountDocument('LS') = 1 AND ^LS52 <> ^TRT475

        Data Elements:
    
            ^TRT475 = TRT:RP:BusinessDetails:Income:Capital.Losses.CarriedForward.Net.Amount
    
            ^LS52 = LS52
            */
        public virtual void VRATOTRT432215()
        {
            bool assertion = childDocument.LS52.GetValueOrDefault() != parentDocument.TRT475.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434087",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in Losses schedule",
                    LongDescription = @"The total of the 'Net capital losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalLossesCarriedForwardNetA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432215" },
                        new ProcessMessageParameter() { Name = "TRT475", Value = parentDocument.TRT475.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS52", Value = childDocument.LS52.GetValueOrDefault().ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432215

        #endregion Validation Rules
    }
}